//
// Utility functions
//
// Copyright (c) 2019 NVIDIA Corporation. All rights reserved.
//
// NVIDIA Corporation and its licensors retain all intellectual property and
// proprietary rights in and to this software, related documentation and any
// modifications thereto. Any use, reproduction, disclosure or distribution
// of this software and related documentation without an express license
// agreement from NVIDIA Corporation is strictly prohibited.

#ifndef UTIL_H
#define UTIL_H

#include "log.h"
#include "constants.h"
#include <map>

namespace NvScistreamSample
{
    // Define Packet struct which is used by the client
    typedef struct {
        // The client's handle for the packet.
        NvSciStreamCookie cookie;
        // The NvSciStream's Handle for the packet
        NvSciStreamPacket handle;
        // An array of elements/buffers in the packet
        NvSciBufObj      *buffers;
        // An array of pre-allocated prefences for this packet.
        NvSciSyncFence    fences[NUM_SYNCOBJS];
    } Packet;

    // Define end point info for ipc channel
    typedef struct {
        // channel name
        char chname[15];
        // NvIPC handle
        NvSciIpcEndpoint endpoint;
        // channel info
        NvSciIpcEndpointInfo info;
        // NvIPC event handle
        NvSciIpcEventNotifier eventHandle;
    } Endpoint;

    // Fill in CPU signaler/waiter attribute list
    inline void setCpuSyncAttrList(
        NvSciSyncAccessPerm cpuPerm,
        NvSciSyncAttrList attrList)
    {
        NvSciSyncAttrKeyValuePair keyValue[2];
        bool cpuSync = true;
        keyValue[0].attrKey = NvSciSyncAttrKey_NeedCpuAccess;
        keyValue[0].value = (void*)&cpuSync;
        keyValue[0].len = sizeof(cpuSync);
        keyValue[1].attrKey = NvSciSyncAttrKey_RequiredPerm;
        keyValue[1].value = (void*)&cpuPerm;
        keyValue[1].len = sizeof(cpuPerm);
        CHECK_NVSCIERR(NvSciSyncAttrListSetAttrs(attrList, keyValue, 2));
    }

    inline void copyElementAttr(
        NvSciStreamElementAttr& dst,
        const NvSciStreamElementAttr& src,
        bool isReconciled)
    {
        dst.index = src.index;
        dst.type = src.type;
        dst.mode = src.mode;
        dst.bufAttr = src.bufAttr;
    }

    inline void freeSyncObjs(
        NvSciSyncObj* syncObjs,
        uint32_t count)
    {
        if (syncObjs != nullptr) {
            for (uint32_t i = 0U; i < count; i++) {
                if (syncObjs[i] != nullptr) {
                    NvSciSyncObjFree(syncObjs[i]);
                }
            }
            free(syncObjs);
            syncObjs = nullptr;
        }
    }

    inline void freeElementAttrs(
        NvSciStreamElementAttr* attrs,
        uint32_t count)
    {
        if (attrs != nullptr) {
            for (uint32_t i = 0U; i < count; i++) {
                if (attrs[i].bufAttr != nullptr) {
                    NvSciBufAttrListFree(attrs[i].bufAttr);
                    attrs[i].bufAttr = nullptr;
                }
            }
            free(attrs);
            attrs = nullptr;
        }
    }

}

#endif